@@ -1,7 +1,7 @@
 /*
  * Object.java
  *
- * Copyright (c) 2006-2007 Operational Dynamics Consulting Pty Ltd
+ * Copyright (c) 2006-2008 Operational Dynamics Consulting Pty Ltd
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -11,12 +11,13 @@
  */
 package org.gnome.glib;
 
-import java.util.HashSet;
+import java.util.ArrayList;
 
 import org.freedesktop.bindings.Constant;
 import org.freedesktop.bindings.Debug;
 import org.freedesktop.bindings.Flag;
 import org.freedesktop.bindings.Proxy;
+import org.gnome.pango.FontDescription;
 
 /**
  * Base class of the object system used by GLib and libraries based on it,
@@ -27,13 +28,13 @@ import org.freedesktop.bindings.Proxy;
  * 
  * <p>
  * <i>Methods here provide the mechanism to get and set "properties" on the
- * underlying <code>GObject</code>s. As a deliberate design decision to
- * ensure type safety, however, these are not exposed for public use. To offer
- * a getter or setter for a property, a java-gnome subclass must expose an
- * explicitly named method. For example, to set the</i> "<code>righteous</code>"
- * <i>property, (assuming that the <code>GObject</code> in question has such
- * a property, that it is writable, and that it takes a string), create a
- * method like this:</i>
+ * underlying <code>GObject</code>s. As a deliberate design decision to ensure
+ * type safety, however, these are not exposed for public use. To offer a
+ * getter or setter for a property, a java-gnome subclass must expose an
+ * explicitly named method. For example, to set the</i> <var>righteous</var>
+ * <i>property, (assuming that the <code>GObject</code> in question has such a
+ * property, that it is writable, and that it takes a string), create a method
+ * like this:</i>
  * 
  * <pre>
  * public void setRighteous(String value) {
@@ -41,9 +42,19 @@ import org.freedesktop.bindings.Proxy;
  * }
  * </pre>
  * 
- * <i>This is of course insanely type-unsafe. In general, luckily, this is not
- * necessary, as most <code>GObject</code>s provide convenience methods for
- * such things, and should be used in preference wherever available.</i>
+ * <i>The use of a String property name is, of course, insanely type-unsafe,
+ * which is why we don't expose it in our public API. Luckily, though, in
+ * general it is not necessary to use this at all, as most
+ * <code>GObject</code>s provide convenience methods for such things, and
+ * should be used in preference wherever available.</i>
+ * 
+ * <p>
+ * <i>It is probably also worth noting that these code paths are less
+ * efficient by about 8:1 as a <code>GValue</code> must be constructed as a
+ * wrapper around the data for the property being set. While that's not too
+ * expensive in C, from Java (and from all other language bindings of GTK) it
+ * requires allocation and initialization of the wrapper structure which
+ * really is just throw away.</i>
  * 
  * @author Andrew Cowie
  * @since 4.0.0
@@ -57,7 +68,7 @@ public abstract class Object extends Proxy
      * a weak Java reference from the BindingsJavaClosure on the JNI side. See
      * bindings_java_signal.c in src/jni.
      */
-    private HashSet handlers;
+    private ArrayList<Signal> handlers;
 
     protected Object(long pointer) {
         super(pointer);
@@ -68,20 +79,6 @@ public abstract class Object extends Proxy
         GObject.addToggleRef(this);
     }
 
-    /**
-     * Take the actions necessary to release our reference to the underlying
-     * GObject, then carry on to
-     * {@link org.freedesktop.bindings.Proxy#finalize() Proxy's finalize()}.
-     */
-    /*
-     * This is a placeholder to remind us of the cleanup actions that will be
-     * necessary, irrespective of the finalizer technique used.
-     */
-    protected void finalize() {
-        release();
-        super.finalize();
-    }
-
     /**
      * Drop our reference count to the underlying GObject.
      * 
@@ -110,7 +107,7 @@ public abstract class Object extends Proxy
      * that.
      */
     protected void setPropertyString(String name, String value) {
-        GObject.setProperty(this, name, new Value(GValue.createValue(value)));
+        GObject.setProperty(this, name, new Value(value));
     }
 
     /**
@@ -123,7 +120,12 @@ public abstract class Object extends Proxy
      * that.
      */
     protected void setPropertyInteger(String name, int value) {
-        GObject.setProperty(this, name, new Value(GValue.createValue(value)));
+        GObject.setProperty(this, name, new Value(value));
+    }
+
+    protected int getPropertyInteger(String name) {
+        Value value = GObject.getProperty(this, name);
+        return GValue.getInteger(value);
     }
 
     /**
@@ -136,9 +138,9 @@ public abstract class Object extends Proxy
      * that.
      */
     protected void setPropertyBoolean(String name, boolean value) {
-        GObject.setProperty(this, name, new Value(GValue.createValue(value)));
+        GObject.setProperty(this, name, new Value(value));
     }
-    
+
     protected boolean getPropertyBoolean(String name) {
         Value value = GObject.getProperty(this, name);
         return GValue.getBoolean(value);
@@ -150,7 +152,7 @@ public abstract class Object extends Proxy
      * @since 4.0.4
      */
     protected void setPropertyFloat(String name, float value) {
-        GObject.setProperty(this, name, new Value(GValue.createValue(value)));
+        GObject.setProperty(this, name, new Value(value));
     }
 
     protected float getPropertyFloat(String name) {
@@ -158,13 +160,22 @@ public abstract class Object extends Proxy
         return GValue.getFloat(value);
     }
 
+    protected void setPropertyDouble(String name, double value) {
+        GObject.setProperty(this, name, new Value(value));
+    }
+
+    protected double getPropertyDouble(String name) {
+        Value value = GObject.getProperty(this, name);
+        return GValue.getDouble(value);
+    }
+
     /**
      * Set a property that takes an Object subclass for its value.
      * 
      * @since 4.0.2
      */
     protected void setPropertyObject(String name, Object value) {
-        GObject.setProperty(this, name, new Value(GValue.createValue(value)));
+        GObject.setProperty(this, name, new Value(value));
     }
 
     protected String getPropertyString(String name) {
@@ -172,6 +183,10 @@ public abstract class Object extends Proxy
         return GValue.getString(value);
     }
 
+    protected void setPropertyEnum(String name, Constant value) {
+        GObject.setProperty(this, name, new Value(value));
+    }
+
     protected Constant getPropertyEnum(String name) {
         Value value = GObject.getProperty(this, name);
         return GValue.getEnum(value);
@@ -198,15 +213,21 @@ public abstract class Object extends Proxy
         return GValue.getObject(value);
     }
 
+    protected void setPropertyBoxed(String name, FontDescription desc) {
+        GObject.setProperty(this, name, new Value(desc));
+    }
+
     /**
      * When a Signal handler is connected, we maintain a strong reference from
      * here. We don't use it for anything, just memory management. Only called
-     * from {@link Plumbing#connectSignal(Object, Signal, Class, String)}.
+     * from Plumbing.connectSignal().
      */
     void addHandler(Signal handler) {
-        if (handlers == null) {
-            handlers = new HashSet();
+        synchronized (this) {
+            if (handlers == null) {
+                handlers = new ArrayList<Signal>(1);
+            }
+            handlers.add(handler);
         }
-        handlers.add(handler);
     }
 }
