@@ -1,7 +1,7 @@
 /*
  * Button.java
  *
- * Copyright (c) 2006-2007 Operational Dynamics Consulting Pty Ltd, and Others
+ * Copyright (c) 2006-2008 Operational Dynamics Consulting Pty Ltd, and Others
  * 
  * The code in this file, and the library it is a part of, are made available
  * to you by the authors under the terms of the "GNU General Public Licence,
@@ -23,10 +23,10 @@ package org.gnome.gtk;
  * Since Button is a Bin it strictly only has one child. Internally, however,
  * it may have both an icon image and some text (which is the look commonly
  * seen in the action buttons in Dialog boxes). You can add such an image to a
- * Button by calling {@link #setImage(Image) setImage()}; this works
- * alongside and with {@link #setLabel(String) setLabel()}. The machinery
- * within Button will manage creating the necessary internal structure
- * (HBoxes, Alignments, etc).
+ * Button by calling {@link #setImage(Image) setImage()}; this works alongside
+ * and with {@link #setLabel(String) setLabel()}. The machinery within Button
+ * will manage creating the necessary internal structure (HBoxes, Alignments,
+ * etc).
  * 
  * @author Andrew Cowie
  * @author Vreixo Formoso
@@ -72,8 +72,7 @@ public class Button extends Bin
      * @since 4.0.0
      */
     public Button(String text) {
-        // TODO replace with GtkButton.createButtonWithMnemonic(text) ?
-        super(GtkButton.createButtonWithLabel(text));
+        super(GtkButton.createButtonWithMnemonic(text));
     }
 
     /**
@@ -96,12 +95,11 @@ public class Button extends Bin
      * Set the text showing in the Button.
      * 
      * <p>
-     * If you created an empty Button without a Label using
-     * {@link #Button() Button()}, this will create a Label nested in an
-     * Alignment for you. That <i>won't</i> work if you create an empty
-     * Button then put a custom Widget in place with
-     * {@link Container#add(Widget) add()} instead of employing this method
-     * and/or {@link #setImage(Image) setImage()}).
+     * If you created an empty Button without a Label using {@link #Button()
+     * Button()}, this will create a Label nested in an Alignment for you.
+     * That <i>won't</i> work if you create an empty Button then put a custom
+     * Widget in place with {@link Container#add(Widget) add()} instead of
+     * employing this method and/or {@link #setImage(Image) setImage()}).
      * 
      * @since 4.0.0
      */
@@ -118,8 +116,7 @@ public class Button extends Bin
      * @since 4.0.0
      */
     public String getLabel() {
-        // return GtkButton.getLabel(this);
-        return super.getPropertyString("label");
+        return GtkButton.getLabel(this);
     }
 
     /**
@@ -209,9 +206,7 @@ public class Button extends Bin
      * @since 4.0.1
      */
     public ReliefStyle getRelief() {
-        // return GtkButton.getRelief(this);
-        // TODO use real translation layer method
-        return (ReliefStyle) getPropertyEnum("relief");
+        return GtkButton.getRelief(this);
     }
 
     /**
@@ -223,59 +218,160 @@ public class Button extends Bin
      * emitted, but only if the cursor is still in the Button when the mouse
      * button is released. You're probably used to this behaviour without
      * realizing it.</i>
-     */
-    public interface CLICKED extends GtkButton.CLICKED
-    {
-        public void onClicked(Button source);
-    }
-
-    /**
-     * Hook up a handler to receive "clicked" events on this Button. A typical
-     * example of how this is used is as follows:
+     * 
+     * <h2>All about signal handling</h2>
+     * 
+     * <h3>Use an anonymous inner class</h3>
+     * 
+     * <p>
+     * In general, the way that java-gnome is intended to be used is for you
+     * to create an anonymous inner class right where you call the
+     * <code>connect()</code> method to hook up the signal. A typical example
+     * is as follows:
      * 
      * <pre>
-     * Button b;
-     *              
-     * b.connect(new Button.CLICKED() {
+     * final Button b;
+     * 
+     * b.connect(new Button.Clicked() {
      *     public void onClicked(Button source) {
-     *         // do something!
+     *         doSomething();
      *     }
      * }
      * </pre>
      * 
-     * <p>
-     * You can of course create a subclass of Button.CLICKED and then use
-     * instances of it if you have highly complicated algorithms to implement.
+     * This is the form we recommend; it has the advantage that your handler
+     * code is close to the code that declares and configures the Widget.
+     * Also, if variables are declared <code>final</code> then you can access
+     * them from within the nested anonymous class, and that makes things easy
+     * indeed.
+     * 
+     * <h3>Using a concrete instance</h3>
      * 
      * <p>
-     * If you implement Button.CLICKED in the class you're currently working
-     * on, then you use a technique called "self-delegation" which can
-     * sometimes work well;
+     * If you have more complex code, perhaps that you need to reuse across
+     * several Widgets, then you can of course create a concrete class that
+     * implements Button.Clicked and then use instances of it if you have
+     * highly complicated algorithms to implement:
+     * 
+     * <pre>
+     * class ComplexHandler implements Button.Clicked
+     * {
+     *     private int field;
+     * 
+     *     ComplexHandler(int param) {
+     *         this.field = param;
+     *     }
+     * 
+     *     public void onClicked(Button source) {
+     *         doSomethingVeryComplicatedWith(field);
+     *     }
+     * }
+     * </pre>
+     * 
+     * and then when you construct the Button:
+     * 
+     * <pre>
+     * b = new Button();
+     * b.connect(new ComplexHandler(42));
+     * </pre>
+     * 
+     * You can just as easily declare one and reuse it if you want to apply
+     * the same logic to a series of Button; this is where the
+     * <code>source</code> parameter that every signal callback signature
+     * includes.
+     * 
+     * <pre>
+     * final ComplexHandler handler;
+     * 
+     * handler = new ComplexHandler(42);
+     * ...
+     * b1.connect(handler);
+     * b2.connect(handler);
+     * b3.connect(handler);
+     * ...
+     * </pre>
+     * 
+     * <h3>Self-delegation</h3>
+     * 
+     * <p>
+     * If you implement Button.Clicked in the class you're currently working
+     * on, then you use a technique called "self-delegation" which can be
+     * useful in certain circumstances:
      * 
      * <pre>
      * b.connect(this);
      * </pre>
      * 
+     * This has disadvantage that you can only have one
+     * <code>onClicked()</code> method in your class, so if you have lots of
+     * Buttons then you lose the benefit of being able to connect different
+     * handlers to different Buttons.
+     * 
+     * <p>
+     * We recommend you use anonymous inner classes in-place whenever
+     * possible. In our experience, having the handler logic next to the code
+     * that declares an instance dramatically improves readability.
+     * 
+     * @author Andrew Cowie
      * @since 4.0.0
      */
+    public interface Clicked extends GtkButton.ClickedSignal
+    {
+        public void onClicked(Button source);
+    }
+
+    /**
+     * Hook up a handler to receive <code>Button.Clicked</code> events on this
+     * Button. See {@link Button.Clicked} for a detailed discussion of how to
+     * connect signals.
+     * 
+     * @since 4.0.0
+     */
+    public void connect(Clicked handler) {
+        GtkButton.connect(this, handler, false);
+    }
+
+    /**
+     * @deprecated
+     */
+    public interface CLICKED extends GtkButton.ClickedSignal
+    {
+    }
+
+    /**
+     * @deprecated
+     */
     public void connect(CLICKED handler) {
-        GtkButton.connect(this, handler);
+        assert false : "use Button.Clicked instead";
+        GtkButton.connect(this, handler, false);
     }
 
     /*
-     * ACTIVATE: "Applications should never connect to this signal, but use
-     * the 'clicked' signal."
-     */
-    /*
-     * ENTERED, PRESSED, etc: "deprecated"
+     * Button.Activate: "Applications should never connect to this signal, but
+     * use the 'clicked' signal."
+     * 
+     * Button.Entered, Button.Pressed, etc: deprecated.
      */
 
     /**
-     * Cause a <code>CLICKED</code> signal to be emitted.
+     * Cause a <code>Button.Clicked</code> signal to be emitted.
      * 
      * @since 4.0.6
      */
     public void emitClicked() {
         GtkButton.clicked(this);
     }
+
+    /**
+     * Set whether clicking the Button will cause the Button to grab the
+     * focus. The default is <code>true</code>, the behaviour you're
+     * accustomed to with regular Buttons. Setting this to <code>false</code>
+     * is only used in unusual situations like toolboxes where you don't want
+     * to steal focus away from where the main action is taking place.
+     * 
+     * @since 4.0.8
+     */
+    public void setFocusOnClick(boolean setting) {
+        GtkButton.setFocusOnClick(this, setting);
+    }
 }
